<?php

namespace SGalinski\SgJobs\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgJobs\Domain\Repository\CompanyRepository;
use SGalinski\SgJobs\Domain\Repository\JobRepository;
use SGalinski\SgJobs\Service\BackendService;
use TYPO3\CMS\Backend\Template\Components\DocHeaderComponent;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * The backend module controller
 */
class BackendController extends ActionController {
	/**
	 * DocHeaderComponent
	 *
	 * @var DocHeaderComponent
	 */
	protected $docHeaderComponent;

	/**
	 * @var CompanyRepository
	 *
	 */
	protected $companyRepository;

	/**
	 * @var JobRepository
	 *
	 */
	protected $jobRepository;

	/**
	 * Inject the CompanyRepository
	 *
	 * @param CompanyRepository $companyRepository
	 */
	public function injectCompanyRepository(CompanyRepository $companyRepository): void {
		$this->companyRepository = $companyRepository;
	}

	/**
	 * Inject the JobRepository
	 *
	 * @param JobRepository $jobRepository
	 */
	public function injectJobRepository(JobRepository $jobRepository): void {
		$this->jobRepository = $jobRepository;
	}

	/**
	 * Show all job offers and options to manage them
	 *
	 * @param array $filters
	 * @return void
	 */
	public function indexAction(array $filters = []): void {
		try {
			$pageUid = (int) GeneralUtility::_GP('id');

			// create docheader + buttons
			$pageInfo = BackendUtility::readPageAccess($pageUid, $GLOBALS['BE_USER']->getPagePermsClause(1));
			if ($pageInfo === FALSE) {
				$pageInfo = ['uid' => $pageUid];
			}

			$this->docHeaderComponent = GeneralUtility::makeInstance(DocHeaderComponent::class);
			$this->docHeaderComponent->setMetaInformation($pageInfo);
			BackendService::makeButtons($this->docHeaderComponent, $this->request);
			$this->view->assign('docHeader', $this->docHeaderComponent->docHeaderContent());
			// get all jobs
			/** @var ObjectStorage $jobs */
			$jobs = $this->jobRepository->findBackendJobs($pageUid, $filters);

			$totalJobCount = \count($jobs);

			$this->view->assign('pages', BackendService::getPagesWithJobRecords());
			$this->view->assign('pageUid', $pageUid);

			$sortingData = [];
			if ($GLOBALS['TYPO3_CONF_VARS']['EXTENSIONS']['sg_jobs']['allowManualSorting'] && count($filters) <= 0) {

				/**
				 *
				 * Somehow the pId is changed so data gets lost.
				 *
				 *
				 * $previousUid = 0;
				 * $sortingData = [];
				 * foreach ($jobs as $job) {
				 * if ($previousUid) {
				 * $sortingData['prev'][$job->getUid()] = $previousUid;
				 * $sortingData['next'][$previousUid] = $job->getUid();
				 * }
				 * $previousUid = $job->getUid();
				 * }
				 */
				$this->view->assign('manualSortingDestroysEverything', TRUE);
			}

			$this->view->assign('sortingData', $sortingData);

			// get all Locations
			/** @noinspection PhpUndefinedMethodInspection */
			/** @var QueryResultInterface $companies */
			$companies = $this->companyRepository->findByPid($pageUid);
			$this->view->assign('locationOptions', $companies);
			if ($totalJobCount || $companies->count()) {
				$this->view->assign('jobs', $jobs);
				$this->view->assign('filters', $filters);
			} else {
				$this->view->assign('noRecords', 1);
				$this->view->assign('isAdmin', $GLOBALS['BE_USER']->isAdmin());
				$this->addFlashMessage(
					LocalizationUtility::translate('backend.notice.noRecords', 'SgJobs'), '', FlashMessage::INFO
				);
			}

		} catch (\Exception $exception) {
			// check for NULL value in view and render an error message
			$this->view->assign('docHeader', NULL);
		}
	}
}
